<?php

return [

  'routes' => [
    'api' => [
      'register' => [
        'enabled' => true,
        'route' => 'api/v1/auth/register',
        'config' => [
          'uses' => '\Myriad\Authentication\Http\Controllers\RegistrationController@register',
          'as' => 'users.register'
        ]
      ],
      'login' => [
        'enabled' => true,
        'route' => 'api/v1/auth/login',
        'config' => [
          'uses' => '\Myriad\Authentication\Http\Controllers\LoginController@login',
          'as' => 'users.login'
        ]
      ],
      'me' => [
        'enabled' => true,
        'route' => 'api/v1/auth/me',
        'config' => [
          'middleware' => 'auth:api',
          'uses' => '\Myriad\Authentication\Http\Controllers\UserController@me',
          'as' => 'users.me'
        ]
      ],
      'verification_codes' => [
        'send' => [
          'enabled' => true,
          'route' => 'api/v1/auth/verification_codes/send',
          'config' => [
            'uses' => '\Myriad\Authentication\Http\Controllers\VerificationCodeController@send',
            'as' => 'verification_codes.send'
          ]
        ],
        'verify' => [
          'enabled' => true,
          'route' => 'api/v1/auth/verification_codes/verify',
          'config' => [
            'uses' => '\Myriad\Authentication\Http\Controllers\VerificationCodeController@verify',
            'as' => 'verification_codes.verify'
          ]
        ]
      ]
    ]
  ],

  'table_names' => [
    'users' => env('AUTHENTICATION_USERS_TABLE_NAME', 'users'),
    'user_phones' => env('AUTHENTICATION_USER_PHONES_TABLE_NAME', 'user_phones'),
    'user_emails' => env('AUTHENTICATION_USER_EMAILS_TABLE_NAME', 'user_emails'),
    'verification_codes' => env('AUTHENTICATION_VERIFICATION_CODES_TABLE_NAME', 'verification_codes'),
    'access_tokens' => env('AUTHENTICATION_ACCESS_TOKENS_TABLE_NAME', 'access_tokens'),
  ],

  'user_model' => Myriad\Authentication\Models\User::class,
  'user_repository' => Myriad\Authentication\Lib\Repositories\EloquentUserRepository::class,
  'user_transformer' => Myriad\Authentication\Http\Transformers\UserTransformer::class,

  'morph_map' => [
    'user' => '\Myriad\Authentication\Models\User'
  ],

  'verification_code_expiration_minutes' => env('VERIFICATION_CODE_EXPIRATION_MINUTES', 5),
  'access_token_expiration_days' => env('AUTHENTICATION_ACCESS_TOKEN_EXPIRATION_DAYS', 365),

  'event_listeners' => [
    'user_registered_with_email' => [
      '\Myriad\Authentication\Lib\Listeners\UserRegisteredWithEmailListener'
    ],
    'user_registered_with_phone' => [
      //TODO: add listener
    ]
  ],

  'email' => [
    'app_name' => 'Application',
    'templates' => [
      'email_address_verification' => [
        'subject' => 'Please verify your email address',
        'template' => 'authentication::email.auth.email_verification'
      ]
    ]
  ],

  'text_messages' => [
    'from_number' => env('TWILIO_ACCOUNT_FROM_NUMBER'),
    'text_message_sender' => [
      'class' => '\Myriad\Authentication\Lib\Services\TwilioTextMessageSender',
      'config' => [
        'sid' => env('TWILIO_ACCOUNT_SID'),
        'auth_token' => env('TWILIO_ACCOUNT_AUTH_TOKEN')
      ]
    ],
    'app_name' => 'Application',
    'templates' => [
      'phone_verification' => [
        'template' => 'authentication::text_messages.auth.phone_verification'
      ]
    ]
  ],

  'verification_strategies' => [
    'email' => '\Myriad\Authentication\Lib\Strategies\EmailVerification',
    'phone' => '\Myriad\Authentication\Lib\Strategies\PhoneNumberVerification',
  ]
];
