# Group Authentication
These routes deal with authentication.

## Register [POST /v1/auth/register]
Start the registration process for a new user. The user must submit either a phone number or an email, but not both. A verification text message will be sent to them if they enter a phone number. Likewise a verification meail message will be sent to them if they submit an email address. Upon successful completion of verification process, a new user will be created.

+ Request (application/json)

    + Body

            {
              "first_name": "george",
              "last_name": "washington",
              "phone": "555-665-4345",
              "type": "personal",
              "password": "IWasTheFirstPrez1776",
              "password_confirmation": "IWasTheFirstPrez1776",
            }

    + Schema

            {
              "$schema": "http://json-schema.org/draft-04/schema#",
              "title": "Registration request",
              "description": "A request to register with the system",
              "type": "object",
              "properties": {
                "first_name": {
                  "type": "string",
                  "description": "The user's first name."
                },
                "last_name": {
                  "type": "string",
                  "description": "The user's last name."
                },
                "phone": {
                  "type": "string",
                  "description": "The user's phone number."
                },
                "email": {
                  "type": "string",
                  "description": "The user's email address."
                },
                "type": {
                  "description": "The type of contact method.",
                  "enum": ["work", "personal"]
                },
                "password": {
                  "type": "string",
                  "description": "The password the user wishes to have."
                },
                "password_confirmation": {
                  "type": "string",
                  "description": "Confirmation of the above password. These must match."
                }
              },
              "required": [
                "first_name",
                "last_name",
                "type",
                "password",
                "password_confirmation"
              ]
            }

+ Response 200 (application/json)

+ Response 422 (application/json)
    + Attributes (object)
        - error (object)
            - code: `VALIDATION_ERROR` (string)
            - message: `You have validation errors in your submission` (string)
            - validation_messages (object)
                - first_name (array[ValidationMessage])
                - last_name (array[ValidationMessage])
                - email (array[ValidationMessage])
                - phone (array[ValidationMessage])
                - type (array[ValidationMessage])
                - password (array[ValidationMessage])
                - password_confirmation (array[ValidationMessage])

## Login [POST /v1/auth/login]
Login with the current user with either an email address or a phone number.

+ Request (application/json)
    + Attributes (object)
        - contact: `george.washington@example.com` (string, required) - The user's email address or phone number.
        - password: `5VphcVNdGhJi` (string, required) - The user's password.

+ Response 200 (application/json)
    + Attributes (object)
        - data (object)
            - Include AccessToken
            - Include TokenExpires

+ Response 401 (application/json)
    + Attributes (object)
        - data (object)
            - code: `AUTHENTICATION_FAILED` (string) - The code indicating authentication failure
            - error: `Invalid Credentials` (string) - The error message indicating invalid credentials
            - more (object)

+ Response 422 (application/json)
    + Attributes (object)
        - error (object)
            - code: `VALIDATION_ERROR` (string)
            - message: `You have validation errors in your submission` (string)
            - validation_messages (object)
                - contact (array[ValidationMessage])
                - password (array[ValidationMessage])

## Me [GET /v1/auth/me]
Get the currently logged in user's information.

+ Response 200 (application/json)
    + Attributes (User)

## Change Password [PUT /v1/auth/password/change]
Change the user's password

+ Request (application/json)
    + Attributes (object)
        - existing_password: `oldpassword` (string, required) - The user's old password
        - new_password: `newpassword` (string, required) - The user's new password
        - new_password_confirmation: `newpassword` (string, required) - The user's new password confirmation. This must match 'new_password'.

+ Response 200 (application/json)

+ Response 401 (application/json)
    + Attributes (object)
        - data (object)
            - code: `INVALID_EXISTING_PASSWORD` (string) - The existing password is invalid
            - error: `Invalid existing password` (string) - Error for existing password invalidity
            - more (object)

## Forgot Password [POST /v1/auth/password/forgot]
Send an email or text message to the user containing a verification code. The verification code can then be used in combination with the email or phone number to [reset the user's password'](#authentication-reset-password).

+ Request (application/json)
    + Attributes (object)
        - contact: `george.washington@gmail.com` (string, required) - The user's email address or cell phone number to send the notification to

+ Response 200 (application/json)

## Reset Password [PUT /v1/auth/password/reset]
Reset the user's password. This endpoint is to be used after calling the [Forgot Password](#authentication-forgot-password) endpoint. The code is the verification code that was sent to the user via either text message or email.

+ Request (application/json)
    + Attributes (object)
        - contact: `george.washington@gmail.com` (string, required) - The user's email address or cell phone number that the verification notification was sent to
        - code: `12345` (string, required) - The password reset code that was sent to the user via their chosen notification method
        - Include Password
        - Include PasswordConfirmation

+ Response 200 (application/json)

+ Response 401 (application/json)
    + Attributes (object)
        - data (object)
            - code: `INVALID_CODE` (string) - The code is invalid
            - error: `Invalid CODE` (string) - Error for invalid code
            - more (object)

+ Response 422 (application/json)
    + Attributes (object)
        - error (object)
            - code: `VALIDATION_ERROR` (string)
            - message: `You have validation errors in your submission` (string)
            - validation_messages (object)
                - code (array[ValidationMessage])
                - contact (array[ValidationMessage])
                - password (array[ValidationMessage])
                - password_confirmation (array[ValidationMessage])


## Send Verification Code [POST /v1/auth/verification_codes/send]
Send a verification code to the user's email address or cell phone number. This code can then be used when making the call to [Verify Verification Code](#authentication-verify-verification-code)

+ Request (application/json)
    + Attributes (object)
        - email: `george.washington@example.com` (string, nullable, optional) - The user's email address to send the notification to.
        - phone: `555-555-5555` (string, nullable, optional) - The user's phone number to send the notification to.

+ Response 200 (application/json)

+ Response 422 (application/json)
    + Attributes (object)
        - error (object)
            - code: `VALIDATION_ERROR` (string)
            - message: `You have validation errors in your submission` (string)
            - validation_messages (object)
                - email (array[ValidationMessage])
                - phone (array[ValidationMessage])

## Verify Verification Code [POST /v1/auth/verification_codes/verify]
This endpoint is to be used after calling the [Send Verification Code](#authentication-send-verification-code) endpoint. Upon the user successfully opening the email message or text message and obtaining the authorization code, the code and contact can be used to authenticate the user to the provided email address or cell phone. The endpoint will respond with an access token and its associated expiration date.

+ Request (application/json)
    + Attributes (object)
        - contact: `george.washington@gmail.com` (string, required) - The user's email address or cell phone number that the verification notification was sent to
        - code: `12345` (string, required) - The password reset code that was sent to the user via their chosen notification method

+ Response 200 (application/json)
    + Attributes (object)
        - data (object)
            - Include AccessToken
            - Include TokenExpires

+ Response 401 (application/json)
    + Attributes (object)
        - data (object)
            - code: `INVALID_CODE` (string) - The code is invalid
            - error: `Invalid CODE` (string) - Error for invalid code
            - more (object)

+ Response 422 (application/json)
    + Attributes (object)
        - error (object)
            - code: `VALIDATION_ERROR` (string)
            - message: `You have validation errors in your submission` (string)
            - validation_messages (object)
                - code (array[ValidationMessage])
                - contact (array[ValidationMessage])

## Data Structures

### Id (object)
- id: `1` (number) - The id of the entity

### ValidationMessage (object)
- message: `The value in this field is not valid.` (string) - The validation error message

### AccessToken (object)
- access_token: `48a4034027374be582b936f7d58043af` (string) - The access token to be used to authenticate future requests

### TokenExpires (object)
- expires: `2016-05-03T11:15:54.950Z` (string, required) - The datetime when the access token will expire

### Email (object)
- email: `george.washington@example.com` (string, required) - User Email address.

### EmailType (object)
- email_type: `personal` (enum[string], required) - The User Email type
    - `work`
    - `personal`

### Phone (object)
- phone: `5556439875` (string, required) - User Email address.

### PhoneType (object)
- phone_type: `personal` (enum[string], required) - The User Phone type
    - `work`
    - `cell`
    - `home`

### FirstName (object)
- first_name: `George` (string, required) - User First Name.

### LastName (object)
- last_name: `Washington` (string, required) - User Last Name.

### Password (object)
- password: `5VphcVNdGhJi` (string, required) - User Password. Between 8 and 64 characters.

### PasswordConfirmation (object)
- password_confirmation: `5VphcVNdGhJi` (string, required) - User Password test.  Must equal property 'password'.

### Primary (object)
- primary: `true` (boolean) - This is user's primary record of its type

### UserEmail (object)
- Include Id
- Include Email
- Include EmailType
- Include Primary

### UserPhone (object)
- Include Id
- Include Phone
- Include PhoneType
- Include Primary

### User (object)
- Include Id
- Include FirstName
- Include LastName
- last_login: `2016-05-03T12:21:25.858Z` (string) - The date of the user's last successful login
- phones (object)
    - data (array[UserPhone])
- emails (object)
    - data (array[UserEmail])
