<?php

namespace Myriad\Authentication\Http\Controllers;

use Illuminate\Routing\Controller;
use Illuminate\Contracts\Bus\Dispatcher;
use Myriad\Authentication\Lib\Jobs\VerifyVerificationCode;
use Myriad\Authentication\Lib\Repositories\IAccessTokenRepository;
use Myriad\Installations\InstallationManager;
use Illuminate\Support\Facades\Validator;
use Illuminate\Contracts\Validation\ValidationException;
use Myriad\Authentication\Lib\Services\IEmailConfirmer;
use Illuminate\Contracts\Auth\Guard;
use Myriad\Authentication\Lib\Services\IPhoneNumberConfirmer;
use Myriad\Authentication\Http\Requests\SendVerificationCodeRequest;
use Myriad\Authentication\Http\Requests\VerifyVerificationCodeRequest;

class VerificationCodeController extends Controller {

  public function send(
    SendVerificationCodeRequest $request,
    Guard $guard,
    IEmailConfirmer $emailConfirmer,
    IPhoneNumberConfirmer $phoneNumberConfirmer
  ) {
    $attributes['primary'] = !$guard->check();

    //TODO: move these to jobs
    if ($request->has('email')) {
      $emailConfirmer->sendConfirmationEmail(
        $request->email,
        $guard->user(),
        $attributes
      );
    } else {
      $phoneDigits = $this->keepOnlyDigits($request->phone);
      $phoneNumberConfirmer->sendConfirmationTextMessage(
        $phoneDigits,
        $guard->user(),
        $attributes
      );
    }
  }

  private function keepOnlyDigits($phone) {
    $matches = [];
    preg_match_all('/[0-9]/', $phone, $matches);

    if (count($matches) === 0) {
      return '';
    }

    return implode($matches[0]);
  }

  public function verify(
    VerifyVerificationCodeRequest $request,
    Dispatcher $busDispatcher,
    IAccessTokenRepository $accessTokenRepository,
    InstallationManager $manager
  ) {
    $contact = $request->contact;
    $code = $request->code;

    $user = $busDispatcher->dispatchNow(new VerifyVerificationCode($contact, $code));

    $installation = $manager->getInstallation();
    $accessToken = $accessTokenRepository->createAccessToken($user, $installation);

    return [
      'data' => [
        'access_token' => $accessToken->token,
        'expires_at' => $accessToken->expires_at->toIso8601String()
      ]
    ];
  }

}
