<?php

namespace Myriad\Authentication\Lib\Guards;

use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Guard;
use Illuminate\Auth\GuardHelpers;
use Illuminate\Auth\TokenGuard;
use Illuminate\Contracts\Auth\UserProvider;
use Illuminate\Http\Request;

class AccessTokenGuard extends TokenGuard
{
  const AUTH_HEADER = 'Authorization';
  const TOKEN_REGEX_PATTERN = '/[tT]oken ([^\s]+)/';
  const TOKEN_MATCH_INDEX = 1;

  /**
   * Create a new authentication guard.
   *
   * @param  \Illuminate\Contracts\Auth\UserProvider  $provider
   * @param  \Illuminate\Http\Request  $request
   * @return void
   */
  public function __construct(UserProvider $provider, Request $request)
  {
    parent::__construct($provider, $request);

    $this->inputKey = 'token';
    $this->storageKey = 'token';
  }


  /**
   * Get the token for the current request.
   *
   * @return string
   */
  protected function getTokenForRequest()
  {
    $authHeader = $this->request->header(self::AUTH_HEADER);

    $matches = [];

    if (preg_match(self::TOKEN_REGEX_PATTERN, $authHeader, $matches)) {
      return $matches[self::TOKEN_MATCH_INDEX];
    }
  }
}
