<?php

namespace Myriad\Authentication\Lib\Jobs;

use Illuminate\Contracts\Validation\Factory;
use Myriad\Authentication\Lib\Repositories\IUserRepository;
use Myriad\Authentication\Lib\Repositories\IAccessTokenRepository;
use Illuminate\Contracts\Validation\ValidationException;
use Myriad\Installations\InstallationManager;
use Illuminate\Contracts\Auth\UserProvider;

class CreateUserInstallationAccessToken
{
  private $request;

  public function __construct($request)
  {
    $this->request = $request;
  }

  public static function getRules() {
    return [
      'user_id' => 'required|integer',
      'installation_uuid' => 'required|string'
    ];
  }

  public static function validate(Factory $validationFactory, array $request) {
    // validate request
    $validator = $validationFactory->make($request, self::getRules());

    if ($validator->fails()) {
      throw new ValidationException($validator);
    }
  }

  public function handle(
    Factory $factory,
    UserProvider $userProvider,
    InstallationManager $installationManager,
    IAccessTokenRepository $accessTokenRepository
  ) {
    self::validate($factory, $this->request);

    $user = $userProvider->retrieveById($this->request['user_id']);
    $installation = $installationManager->findInstallationByUUID($this->request['installation_uuid']);
    return $accessTokenRepository->createAccessToken($user, $installation);
  }
}
