<?php

namespace Myriad\Authentication\Lib\Jobs;

use Illuminate\Contracts\Validation\ValidationException;
use Illuminate\Contracts\Validation\Factory;
use Myriad\Authentication\Lib\Repositories\IVerificationCodeRepository;
use Illuminate\Contracts\Auth\Guard;

class VerifyVerificationCode
{
  private $contact;
  private $code;


  /**
   * Create a new instance of VerifyVerificationCode job
   *
   * @param string $contact - The contact the verification code was sent to
   * @param string $code - The verification code that was sent to contact
   */
  public function __construct($contact, $code) {
    $this->contact = $contact;
    $this->code = $code;
  }

  public function handle(
    Factory $factory,
    IVerificationCodeRepository $verificationCodeRepository,
    Guard $guard
  ) {
    $codes = $verificationCodeRepository->findVerificationCodes($this->contact);

    $matchingCode = $this->findMatch($codes);

    if (!$matchingCode) {
      throw new \Exception("No valid code found matching " . $this->code);
    }

    $user = $matchingCode->performVerificationStrategy($this->code);
    $guard->setUser($user);

    return $user;
  }


  private function findMatch($codes) {
    foreach ($codes as $code) {
      if ($code->attemptToMatch($this->code)) {
        return $code;
      }
    }
  }
 
}
