<?php

namespace Myriad\Authentication\Lib\Repositories;

use Myriad\Authentication\Lib\Repositories\IAccessTokenRepository;
use Myriad\Authentication\Models\AccessToken;
use Carbon\Carbon;
use Myriad\Authentication\Models\IAccessTokenOwner;
use Myriad\Authentication\Models\IAccessTokenClient;

class EloquentAccessTokenRepository implements IAccessTokenRepository
{
  public function createAccessToken($owner, $client) {
    $token = $this->generateToken();

    $tokenLifetime = config('authentication.access_token_expiration_days', 365);
    $expiresAt = Carbon::now()->addDays($tokenLifetime);

    return AccessToken::create([
      'token' => $token,
      'expires_at' => $expiresAt,
      'owner_type' => $owner->getMorphClass(),
      'owner_id' => $owner->id,
      'client_type' => $client->getMorphClass(),
      'client_id' => $client->id
    ]);
  }

  public function getUserFromToken($token) {
    $accessToken = $this->getAccessTokenByToken($token);

    if (is_null($accessToken)) {
      return null;
    }

    return $accessToken->owner;
  }

  public function getAccessTokenByToken($token) {
    return AccessToken::where('token', $token)->first();
  }

  private function generateToken($bytes = 96) {
    $rand = random_bytes($bytes);
    $rand = base64_encode($rand);
    $rand = str_replace(['/', '+', '='], ['_', '-', '*'], $rand); // replace non-url safe characters
    return $rand;
  }
}
