<?php


namespace Myriad\Authentication\Models;

trait LookupByEmailOrPhone {

  /**
   * Get all the user's phone numbers
   *
   * @return \Illuminate\Database\Eloquent\Relations\HasMany
   */
  public function phoneNumbers()
  {
    return $this->hasMany(UserPhone::class);
  }

  /**
   * Get all the user's email addresses
   *
   * @return \Illuminate\Database\Eloquent\Relations\HasMany
   */
  public function emailAddresses()
  {
    return $this->hasMany(UserEmail::class);
  }

  public function lookup(array $data) {
    $user = $this->tryRetrieveByEmail($data);

    if (is_null($user)) {
      $user = $this->tryRetrieveByPhone($data);
    }

    return $user;
  }

  protected function tryRetrieveByEmail(array $data) {
    if (!$this->containsEmail($data)) {
      return null;
    }

    return $this->newQuery()->whereHas('emailAddresses', function($query) use ($data) {
      return $query->where('email', $data['email'])
                   ->where('primary', true);
    })->first();
  }

  protected function tryRetrieveByPhone(array $data) {
    if (!$this->containsPhone($data)) {
      return null;
    }

    return $this->newQuery()->whereHas('phoneNumbers', function($query) {
      return $query->where('phone', $data['phone'])
                   ->where('primary', true);
    })->first();
  }


  /**
   * Is there a non-null email in the data array
   *
   * @param array $data
   */
  protected function containsEmail(array $data) {
    return $this->dataContainsKey($data, 'email');
  }

  /**
   * Is there a non-null phone in the data array
   *
   * @param array $data
   */
  protected function containsPhone(array $data) {
    return $this->dataContainsKey($data, 'phone');
  }

  /**
   * Does the data contain a non-null value for key
   *
   * @param array $data
   * @param mixed $key
   */
  protected function dataContainsKey(array $data, $key) {
    if (empty($data)) {
      return false;
    }

    if (!array_key_exists($key, $data) || is_null($data[$key])) {
      return false;
    }

    return true;
  }
}
